<?php

namespace Drupal\firebase_ui\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Component\Datetime\Time;
use Drupal\firebase_ui\Entity\FirebaseNotification;
use Drupal\firebase_ui\Service\NotificationSenderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes Firebase notifications.
 *
 * @QueueWorker(
 *   id = "firebase_ui_notification_queue",
 *   title = @Translation("Firebase UI Notification Queue"),
 *   cron = {"time" = 30}
 * )
 */
class FirebaseNotificationQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  protected EntityTypeManagerInterface $entityTypeManager;
  protected Time $time;
  protected QueueFactory $queueFactory;
  protected NotificationSenderInterface $notificationSender;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, Time $time, QueueFactory $queue_factory, NotificationSenderInterface $notification_sender) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->time = $time;
    $this->queueFactory = $queue_factory;
    $this->notificationSender = $notification_sender;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('datetime.time'),
      $container->get('queue'),
      $container->get('firebase_ui.notification_sender')
    );
  }

  public function processItem($data) {
    $storage = $this->entityTypeManager->getStorage('firebase_notification');
    $notification = $storage->load($data['nid']);

    if (!$notification || !$notification->get('status')->value) return;

    $current_time = $this->time->getCurrentTime();
    if ($notification->get('send_time')->value > $current_time) {
      $this->queueFactory->get('firebase_ui_notification_queue')->createItem(['nid' => $notification->id()]);
      return;
    }

    $users = $this->getTargetUsers($notification);
    foreach ($users as $user) {
      foreach ($user->get('field_firebase_tokens')->getValue() as $tokenItem) {
        $this->notificationSender->sendNotification(
          $tokenItem['value'],
          $notification->get('title')->value,
          $notification->get('message')->value,
          ['type' => 'alert', 'user_id' => $user->id()]
        );
      }
    }
  }

  /**
   * Gets all applicable users for the notification.
   *
   * @param \Drupal\firebase_ui\Entity\FirebaseNotification $notification
   *   The Firebase notification entity.
   *
   * @return \Drupal\user\UserInterface[]
   *   An array of loaded user entities.
   */
  private function getTargetUsers(FirebaseNotification $notification) {
    $type = $notification->get('type')->value;
    $target_ids = array_filter(explode(',', $notification->get('target_ids')->value));
    $uids = [];

    if ($type === 'all') {
      $all_uids = $this->entityTypeManager->getStorage('user')->getQuery()
        ->accessCheck(FALSE)
        ->condition('status', 1)
        ->execute();

      $users = $this->entityTypeManager->getStorage('user')->loadMultiple($all_uids);
      foreach ($users as $user) {
        if (!$user->get('field_firebase_tokens')->isEmpty()) {
          $uids[] = $user->id();
        }
      }
    }

    if ($type === 'specific_users') {
      $uids = $target_ids;
    }

    if ($type === 'roles') {
      $query = $this->entityTypeManager->getStorage('user')->getQuery()
        ->accessCheck(FALSE)
        ->condition('status', 1);
      $or = $query->orConditionGroup();
      foreach ($target_ids as $role) {
        $or->condition('roles', $role);
      }
      $uids = $query->condition($or)->execute();
    }

    return !empty($uids)
      ? $this->entityTypeManager->getStorage('user')->loadMultiple($uids)
      : [];
  }

}
